<?php
namespace XLite\Module\Foridev\TrustPay\Model\Payment\Processor;

use Trustpay\Order;
use Trustpay\Payment;
use Trustpay\PaymentCard;
use Trustpay\PaymentType;
use Trustpay\PaymentWire;
use Trustpay\ResultCodes;

class TrustPay extends \XLite\Model\Payment\Base\Online {
    
    private $card_accepted_currencies = [
        'AUD',
        'CAD',
        'CZK',
        'DKK',
        'EUR',
        'GBP',
        'HRK',
        'HUF',
        'JPY',
        'NOK',
        'PLN',
        'RON',
        'SEK',
        'USD',
    ];
    
    public function __construct()
    {
        $this->loadLib();
    }

    protected function doInitialPayment() {
        \XLite\Core\Session::getInstance()->set('trust_pay_transaction_id', $this->transaction->getPublicTxnId());
        return static::PROLONGATION;
    }

    public function processReturn(\XLite\Model\Payment\Transaction $transaction) {
        parent::processReturn($transaction);
        $this->transaction->setStatus($transaction::STATUS_PENDING);
    }

    public function processCallback(\XLite\Model\Payment\Transaction $transaction)
    {
        parent::processCallback($transaction);

        $request = \XLite\Core\Request::getInstance();

        if($response = $this->initResponse($request)) {
            if($response->validateSignature()) {
                if($status = $this->getStatus($response)) {
                    $this->transaction
                        ->setStatus($status)
                        ->registerTransactionInOrderHistory('callback');
                }
            }
        }
    }

    protected function initResponse($request)
    {
        $response = false;

        if($request->paymentMethod) {
            $class = false;

            if($request->paymentMethod == Payment::ENDPOINT_WIRE) {
                $class = '\Trustpay\ResponseWire';
            } elseif($request->paymentMethod == Payment::ENDPOINT_CARD) {
                $class = '\Trustpay\ResponseCard';
            }

            if($class) {
                $method = \XLite\Core\Database::getRepo('XLite\Model\Payment\Method')
                    ->findOneBy(array('service_name' => 'TrustPay'));

                $response = new $class($this->getSecretKey($method));
                $response->initData($_GET);
            }
        }

        return $response;
    }

    protected function getStatus($response)
    {
        $status = false;

        if($this->checkSuccess($response->getResultCode())) {
            $status = \XLite\Model\Payment\Transaction::STATUS_SUCCESS;

        } elseif($this->checkInProgress($response->getResultCode())) {
            $status = \XLite\Model\Payment\Transaction::STATUS_INPROGRESS;
        }

        return $status;
    }

    protected function checkSuccess($resultCode) {
        $request = \XLite\Core\Request::getInstance();
        return $resultCode == ResultCodes::SUCCESS || ($resultCode == ResultCodes::AUTHORIZED && $request->paymentMethod == Payment::ENDPOINT_CARD);
    }

    protected function checkInProgress($resultCode) {
        $request = \XLite\Core\Request::getInstance();
        return $resultCode == ResultCodes::PROCESSING || ($resultCode == ResultCodes::AUTHORIZED && $request->paymentMethod == Payment::ENDPOINT_WIRE);
    }

    public function getUrl(\XLite\Model\Cart $cart) {
        try {
            $method = \XLite\Core\Database::getRepo('XLite\Model\Payment\Method')
                ->findOneBy(array('service_name' => 'TrustPay'));

            $paymentMethod = $this->getPaymentMethod();

            if(!$paymentMethod) {
                throw new \Exception('Undefined payment method.');
            }
            
            if($paymentMethod == 'wire') {
                if($this->getCurrency($method) != $cart->getCurrency()->getCode()) {
                    throw new \Exception('Currency not supported.aaa');
                }
            }
            else {
                // card
                if(!in_array($cart->getCurrency()->getCode(), $this->card_accepted_currencies)) {
                    throw new \Exception('Currency not supported.eee');
                }
            }

            $trustPayOrder = new Order();
            $trustPayOrder->setId($cart->getOrderId());
            $trustPayOrder->setCurrency($cart->getCurrency()->getCode());
            $trustPayOrder->setTotal($cart->getTotal());

            if($paymentMethod == 'wire') {
                $payment = new PaymentWire($this->getAccountId($method), $this->getSecretKey($method));
            } else {
                $payment = new PaymentCard($this->getAccountId($method), $this->getSecretKey($method));

                $transaction = \XLite\Core\Database::getRepo('XLite\Model\Payment\Transaction')
                    ->findOneByPublicTxnId(\XLite\Core\Session::getInstance()->get('trust_pay_transaction_id'));

                if(!$transaction) {
                    throw new \Exception('Transaction not found.');
                }

                $trustPayOrder->setBillingCity($transaction->getOrder()->getProfile()->getBillingAddress()->getCity());
                $trustPayOrder->setBillingCountry($transaction->getOrder()->getProfile()->getBillingAddress()->getCountry()->getCode());
                $trustPayOrder->setBillingPostcode($transaction->getOrder()->getProfile()->getBillingAddress()->getZipcode());
                $trustPayOrder->setBillingStreet($transaction->getOrder()->getProfile()->getBillingAddress()->getStreet());
                $trustPayOrder->setCardHolder($transaction->getOrder()->getProfile()->getBillingAddress()->getName());
                $trustPayOrder->setEmail($transaction->getOrder()->getProfile()->getLogin());
            }

            $payment->setOrder($trustPayOrder)
                ->setSandbox($this->isTestMode($method))
                ->setNotificationUrl(
                    $this->getCallbackURL('transactionID') . '&transactionID=' . \XLite\Core\Session::getInstance()->get('trust_pay_transaction_id') . '&paymentMethod=' . $paymentMethod
                )
                ->setReturnUrl(
                    $this->getReturnURL('transactionID') . '&transactionID=' . \XLite\Core\Session::getInstance()->get('trust_pay_transaction_id')
                )
                ->setCancelUrl(
                    \XLite::getInstance()->getShopURL(
                        \XLite\Core\Converter::buildURL('checkout', '', array(), \XLite::getCustomerScript()),
                        \XLite\Core\Config::getInstance()->Security->customer_security
                    )
                )
                ->setErrorUrl(
                    \XLite::getInstance()->getShopURL(
                        \XLite\Core\Converter::buildURL('trust_pay', 'error', array(), \XLite::getCustomerScript()),
                        \XLite\Core\Config::getInstance()->Security->customer_security
                    )
                )
                ->setLocalization($this->getLocale($method))
                ->setPaymentMethod($paymentMethod);

            echo json_encode(
                array(
                    'status' => true,
                    'url' => $payment->getUrl(PaymentType::PURCHASE)
                )
            );
        } catch (\Exception $e) {
            echo json_encode(
                array(
                    'status' => false,
                    'error' => $e->getMessage()
                )
            );
        }

        exit;
    }

    protected function getPaymentMethod() {
        $request = \XLite\Core\Request::getInstance();

        $method = false;

        if(
            !empty($request->trustpay_method) &&
            in_array($request->trustpay_method,
                [
                    Payment::ENDPOINT_WIRE,
                    Payment::ENDPOINT_CARD
                ]
            )
        ) {
            $method = $request->trustpay_method;
        }

        return $method;
    }

    public function getInputTemplate()
    {
        return 'modules/Foridev/TrustPay/payment.twig';
    }

    public function getSettingsWidget()
    {
        return 'modules/Foridev/TrustPay/config.twig';
    }

    public function getAdminIconURL(\XLite\Model\Payment\Method $method)
    {
        return true;
    }

    public function isTestMode(\XLite\Model\Payment\Method $method)
    {
        return \XLite\View\FormField\Select\TestLiveMode::TEST === $method->getSetting('sandbox');
    }
    
    protected function loadLib() {
        $lib_files = array(
            'Order',
            'Payment',
            'PaymentType',
            'PaymentWire',
            'PaymentCard',
            'Response',
            'ResponseCard',
            'ResponseWire',
            'ResultCodes',
            'Signature',
        );

        foreach ($lib_files as $file) {
            require_once LC_DIR_MODULES. 'Foridev' . LC_DS . 'TrustPay' . LC_DS . 'lib' . LC_DS . $file . '.php';
        }
    }

    private function getAccountId(\XLite\Model\Payment\Method $method) {
        return $this->isTestMode($method) ? $method->getSetting('test_account_id') : $method->getSetting('live_account_id');
    }

    private function getSecretKey(\XLite\Model\Payment\Method $method) {
        return $this->isTestMode($method) ? $method->getSetting('test_secret_key') : $method->getSetting('live_secret_key');
    }

    private function getLocale(\XLite\Model\Payment\Method $method) {
        return $method->getSetting('locale');
    }

    private function getCurrency(\XLite\Model\Payment\Method $method) {
        return $method->getSetting('currency');
    }
}